@@ -2,12 +2,7 @@
 #include "dijkstra.h"
 #include "heaps/heap_lib.h"
 
-#include <algorithm> // std::fill, std::reverse
-
-#include <Rcpp.h>
-
-const float INFINITE_FLOAT =  std::numeric_limits<float>::max ();
-const float INFINITE_INT =  std::numeric_limits<int>::max ();
+#include "run_sp.h"
 
 template <typename T>
 void inst_graph (DGraph *g, unsigned int nedges,
@@ -24,12 +19,6 @@ void inst_graph (DGraph *g, unsigned int nedges,
         g->addNewEdge (fromi, toi, dist [i], wt [i]);
     }
 }
-template void inst_graph <float> (DGraph *g, unsigned int nedges,
-        std::map <std::string, unsigned int> &vert_map,
-        std::vector <std::string> &from,
-        std::vector <std::string> &to,
-        std::vector <float> &dist,
-        std::vector <float> &wt);
 
 Dijkstra * dijkstra_bheap (unsigned int nverts)
 {
@@ -83,37 +72,40 @@ Rcpp::NumericMatrix rcpp_get_sp_dists (Rcpp::DataFrame graph,
         std::vector <int> toi,
         std::string heap_type)
 {
+    Rcpp::NumericVector id_vec;
     if (fromi [0] < 0) // use all vertices
     {
-        Rcpp::NumericVector id_vec = vert_map_in ["id"];
-        fromi = Rcpp::as <std::vector <int>> (id_vec);
+        id_vec = vert_map_in ["id"];
+        fromi = Rcpp::as <std::vector <int> > (id_vec);
     }
     if (toi [0] < 0) // use all vertices
     {
-        Rcpp::NumericVector id_vec = vert_map_in ["id"];
-        toi = Rcpp::as <std::vector <int>> (id_vec);
+        if (id_vec.size() == 0)
+            id_vec = vert_map_in ["id"];
+        toi = Rcpp::as <std::vector <int> > (id_vec);
     }
-    unsigned int nfrom = fromi.size (), nto = toi.size ();
+    size_t nfrom = fromi.size (), nto = toi.size ();
 
     std::vector <std::string> from = graph ["from"];
     std::vector <std::string> to = graph ["to"];
-    std::vector <float> dist = graph ["d"];
-    std::vector <float> wt = graph ["w"];
+    std::vector <double> dist = graph ["d"];
+    std::vector <double> wt = graph ["w"];
 
-    unsigned int nedges = graph.nrow ();
+    unsigned int nedges = static_cast <unsigned int> (graph.nrow ());
     std::map <std::string, unsigned int> vert_map;
     std::vector <std::string> vert_map_id = vert_map_in ["vert"];
     std::vector <unsigned int> vert_map_n = vert_map_in ["id"];
-    for (int i = 0; i < vert_map_in.nrow (); ++i)
+    for (unsigned int i = 0;
+            i < static_cast <unsigned int> (vert_map_in.nrow ()); ++i)
     {
         vert_map.emplace (vert_map_id [i], vert_map_n [i]);
     }
-    unsigned int nverts = vert_map.size ();
+    unsigned int nverts = static_cast <unsigned int> (vert_map.size ());
 
     DGraph *g = new DGraph (nverts);
     inst_graph (g, nedges, vert_map, from, to, dist, wt);
 
-    Dijkstra *dijkstra = NULL;
+    Dijkstra *dijkstra = std::nullptr_t ();
 
     if (heap_type == "FHeap")
         dijkstra = dijkstra_fheap (nverts);
@@ -128,8 +120,8 @@ Rcpp::NumericMatrix rcpp_get_sp_dists (Rcpp::DataFrame graph,
     else if (heap_type == "Radix")
         dijkstra = dijkstra_radix (nverts);
 
-    float* w = new float [nverts];
-    float* d = new float [nverts];
+    double* w = new double [nverts];
+    double* d = new double [nverts];
     int* prev = new int [nverts];
 
     dijkstra->init (g); // specify the graph
@@ -137,15 +129,16 @@ Rcpp::NumericMatrix rcpp_get_sp_dists (Rcpp::DataFrame graph,
     // initialise dout matrix to NA
     Rcpp::NumericVector na_vec = Rcpp::NumericVector (nfrom * nto,
             Rcpp::NumericVector::get_na ());
-    Rcpp::NumericMatrix dout (nfrom, nto, na_vec.begin ());
+    Rcpp::NumericMatrix dout (static_cast <int> (nfrom),
+            static_cast <int> (nto), na_vec.begin ());
     for (unsigned int v = 0; v < nfrom; v++)
     {
-        std::fill (w, w + nverts, INFINITE_FLOAT);
-        std::fill (d, d + nverts, INFINITE_FLOAT);
+        std::fill (w, w + nverts, INFINITE_DOUBLE);
+        std::fill (d, d + nverts, INFINITE_DOUBLE);
 
-        dijkstra->run (d, w, prev, fromi [v]);
+        dijkstra->run (d, w, prev, static_cast <unsigned int> (fromi [v]));
         for (unsigned int vi = 0; vi < nto; vi++)
-            if (w [toi [vi]] < INFINITE_FLOAT)
+            if (w [toi [vi]] < INFINITE_DOUBLE)
                 dout (v, vi) = d [toi [vi]];
     }
 
@@ -183,37 +176,40 @@ Rcpp::List rcpp_get_paths (Rcpp::DataFrame graph,
         std::vector <int> toi,
         std::string heap_type)
 {
+    Rcpp::NumericVector id_vec;
     if (fromi [0] < 0) // use all vertices
     {
-        Rcpp::NumericVector id_vec = vert_map_in ["id"];
-        fromi = Rcpp::as <std::vector <int>> (id_vec);
+        id_vec = vert_map_in ["id"];
+        fromi = Rcpp::as <std::vector <int> > (id_vec);
     }
     if (toi [0] < 0) // use all vertices
     {
-        Rcpp::NumericVector id_vec = vert_map_in ["id"];
-        toi = Rcpp::as <std::vector <int>> (id_vec);
+        if (id_vec.size() == 0)
+            id_vec = vert_map_in ["id"];
+        toi = Rcpp::as <std::vector <int> > (id_vec);
     }
-    unsigned int nfrom = fromi.size (), nto = toi.size ();
+    size_t nfrom = fromi.size (), nto = toi.size ();
 
     std::vector <std::string> from = graph ["from"];
     std::vector <std::string> to = graph ["to"];
-    std::vector <float> dist = graph ["d"];
-    std::vector <float> wt = graph ["w"];
+    std::vector <double> dist = graph ["d"];
+    std::vector <double> wt = graph ["w"];
 
-    unsigned int nedges = graph.nrow ();
+    unsigned int nedges = static_cast <unsigned int> (graph.nrow ());
     std::map <std::string, unsigned int> vert_map;
     std::vector <std::string> vert_map_id = vert_map_in ["vert"];
     std::vector <unsigned int> vert_map_n = vert_map_in ["id"];
-    for (int i = 0; i < vert_map_in.nrow (); ++i)
+    for (unsigned int i = 0;
+            i < static_cast <unsigned int> (vert_map_in.nrow ()); ++i)
     {
         vert_map.emplace (vert_map_id [i], vert_map_n [i]);
     }
-    unsigned int nverts = vert_map.size ();
+    unsigned int nverts = static_cast <unsigned int> (vert_map.size ());
 
     DGraph *g = new DGraph (nverts);
     inst_graph (g, nedges, vert_map, from, to, dist, wt);
 
-    Dijkstra *dijkstra = NULL;
+    Dijkstra *dijkstra = std::nullptr_t ();
 
     if (heap_type == "FHeap")
         dijkstra = dijkstra_fheap (nverts);
@@ -231,30 +227,32 @@ Rcpp::List rcpp_get_paths (Rcpp::DataFrame graph,
     dijkstra->init (g); // specify the graph
 
     Rcpp::List res (nfrom);
-    float* w = new float [nverts];
-    float* d = new float [nverts];
+    double* w = new double [nverts];
+    double* d = new double [nverts];
     int* prev = new int [nverts];
 
     for (unsigned int v = 0; v < nfrom; v++)
     {
-        std::fill (w, w + nverts, INFINITE_FLOAT);
-        std::fill (d, d + nverts, INFINITE_FLOAT);
+        std::fill (w, w + nverts, INFINITE_DOUBLE);
+        std::fill (d, d + nverts, INFINITE_DOUBLE);
 
-        dijkstra->run (d, w, prev, fromi [v]);
+        dijkstra->run (d, w, prev, static_cast <unsigned int> (fromi [v]));
 
         Rcpp::List res1 (nto);
         for (unsigned int vi = 0; vi < nto; vi++)
         {
             std::vector <unsigned int> onePath;
-            if (w [toi [vi]] < INFINITE_FLOAT)
+            if (w [toi [vi]] < INFINITE_DOUBLE)
             {
-                unsigned int target = toi [vi];
+                int target = toi [vi];
                 while (target < INFINITE_INT)
                 {
                     // Note that targets are all C++ 0-indexed and are converted
                     // directly here to R-style 1-indexes.
-                    onePath.push_back (target + 1);
+                    onePath.push_back (static_cast <unsigned int> (target + 1));
                     target = prev [target];
+                    if (target < 0)
+                        break;
                 }
             }
             std::reverse (onePath.begin (), onePath.end ());
@@ -294,33 +292,36 @@ Rcpp::NumericVector rcpp_aggregate_flows (Rcpp::DataFrame graph,
         Rcpp::NumericMatrix flows,
         std::string heap_type)
 {
+    Rcpp::NumericVector id_vec;
     if (fromi [0] < 0) // use all vertices
     {
-        Rcpp::NumericVector id_vec = vert_map_in ["id"];
-        fromi = Rcpp::as <std::vector <int>> (id_vec);
+        id_vec = vert_map_in ["id"];
+        fromi = Rcpp::as <std::vector <int> > (id_vec);
     }
     if (toi [0] < 0) // use all vertices
     {
-        Rcpp::NumericVector id_vec = vert_map_in ["id"];
-        toi = Rcpp::as <std::vector <int>> (id_vec);
+        if (id_vec.size() == 0)
+            id_vec = vert_map_in ["id"];
+        toi = Rcpp::as <std::vector <int> > (id_vec);
     }
-    unsigned int nfrom = fromi.size (), nto = toi.size ();
+    size_t nfrom = fromi.size (), nto = toi.size ();
 
     std::vector <std::string> from = graph ["from"];
     std::vector <std::string> to = graph ["to"];
-    std::vector <float> dist = graph ["d"];
-    std::vector <float> wt = graph ["w"];
+    std::vector <double> dist = graph ["d"];
+    std::vector <double> wt = graph ["w"];
 
-    unsigned int nedges = graph.nrow ();
+    unsigned int nedges = static_cast <unsigned int> (graph.nrow ());
     std::vector <std::string> vert_name = vert_map_in ["vert"];
     std::vector <unsigned int> vert_indx = vert_map_in ["id"];
     // Make map from vertex name to integer index
     std::map <std::string, unsigned int> vert_map_i;
-    for (int i = 0; i < vert_map_in.nrow (); ++i)
+    for (unsigned int i = 0;
+            i < static_cast <unsigned int> (vert_map_in.nrow ()); ++i)
     {
         vert_map_i.emplace (vert_name [i], vert_indx [i]);
     }
-    unsigned int nverts = vert_map_i.size ();
+    unsigned int nverts = static_cast <unsigned int> (vert_map_i.size ());
 
     /* Flows from the dijkstra output are reallocated based on matching vertex
      * pairs to edge indices. Note, however, that contracted graphs frequently
@@ -329,7 +330,7 @@ Rcpp::NumericVector rcpp_aggregate_flows (Rcpp::DataFrame graph,
      * pairs, and the other to hold minimal distances.
      */
     std::unordered_map <std::string, unsigned int> verts_to_edge_map;
-    std::unordered_map <std::string, float> verts_to_dist_map;
+    std::unordered_map <std::string, double> verts_to_dist_map;
     for (unsigned int i = 0; i < from.size (); i++)
     {
         std::string two_verts = "f" + from [i] + "t" + to [i];
@@ -346,7 +347,7 @@ Rcpp::NumericVector rcpp_aggregate_flows (Rcpp::DataFrame graph,
     DGraph *g = new DGraph (nverts);
     inst_graph (g, nedges, vert_map_i, from, to, dist, wt);
 
-    Dijkstra *dijkstra = NULL;
+    Dijkstra *dijkstra = std::nullptr_t ();
 
     if (heap_type == "FHeap")
         dijkstra = dijkstra_fheap (nverts);
@@ -364,8 +365,8 @@ Rcpp::NumericVector rcpp_aggregate_flows (Rcpp::DataFrame graph,
     dijkstra->init (g); // specify the graph
 
     Rcpp::List res (nfrom);
-    float* w = new float [nverts];
-    float* d = new float [nverts];
+    double* w = new double [nverts];
+    double* d = new double [nverts];
     int* prev = new int [nverts];
 
     unsigned int junk = 0; // TODO: Delete that!
@@ -373,10 +374,10 @@ Rcpp::NumericVector rcpp_aggregate_flows (Rcpp::DataFrame graph,
     Rcpp::NumericVector aggregate_flows (from.size ()); // 0-filled by default
     for (unsigned int v = 0; v < nfrom; v++)
     {
-        std::fill (w, w + nverts, INFINITE_FLOAT);
-        std::fill (d, d + nverts, INFINITE_FLOAT);
+        std::fill (w, w + nverts, INFINITE_DOUBLE);
+        std::fill (d, d + nverts, INFINITE_DOUBLE);
 
-        dijkstra->run (d, w, prev, fromi [v]);
+        dijkstra->run (d, w, prev, static_cast <unsigned int> (fromi [v]));
 
         Rcpp::List res1 (nto);
         for (unsigned int vi = 0; vi < nto; vi++)
@@ -384,28 +385,29 @@ Rcpp::NumericVector rcpp_aggregate_flows (Rcpp::DataFrame graph,
             if (vi != v) // Exclude self-flows
             {
                 std::vector <unsigned int> onePath;
-                float flow_ij = flows (v, vi);
+                double flow_ij = flows (v, vi);
                 if (flow_ij > 0)
                     junk++;
-                if (w [toi [vi]] < INFINITE_FLOAT)
+                if (w [toi [vi]] < INFINITE_DOUBLE)
                 {
                     // target values are int indices into vert_map_in, which means
                     // corresponding vertex IDs can be taken directly from
                     // vert_name
-                    unsigned int target = toi [vi];
+                    int target = toi [vi];
                     while (target < INFINITE_INT)
                     {
                         if (prev [target] >= 0 && prev [target] < INFINITE_INT)
                         {
-                            std::string v2 = "f" + vert_name [prev [target]] +
-                                "t" + vert_name [target];
+                            std::string v2 = "f" +
+                                vert_name [static_cast <size_t> (prev [target])] +
+                                "t" + vert_name [static_cast <size_t> (target)];
                             aggregate_flows [verts_to_edge_map.at (v2)] += flow_ij;
                         }
 
                         target = prev [target];
                         // Only allocate that flow from origin vertex v to all
                         // previous vertices up until the target vi
-                        if (target == fromi [v])
+                        if (target < 0 || target == fromi [v])
                         {
                             break;
                         }
